# Generated by Django 4.2.27 on 2026-01-25 09:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('charlink', '0003_compliancefilter'),
    ]

    operations = [
        migrations.AlterField(
            model_name='appsettings',
            name='default_selection',
            field=models.BooleanField(),
        ),
    ]
