{% load django_bootstrap5 %}
{% load i18n %}

<div id="charlink-dashboard-widget" class="col-12 mb-3">
    <div class="card px-2">
        <div class="card-body text-center">
            {% include "framework/dashboard/widget-title.html" with title="CharLink" %}
            <div class="row justify-content-center">
                <div class="col-lg-4">
                    <form method="post" action="{% url "charlink:dashboard_post" %}">
                        {% csrf_token %}
                        {% bootstrap_form form %}
                        <button type="submit" class="btn btn-outline-info mt-2">{% translate "Login" %}</button>
                    </form>
                </div>
            </div>
        </div>
    </div>
</div>
