# Generated by Django 4.2.23 on 2025-08-22 10:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('charlink', '0002_appsettings_alter_general_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='ComplianceFilter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('negate', models.BooleanField(default=False)),
                ('selected_apps', models.ManyToManyField(related_name='+', to='charlink.appsettings')),
            ],
            options={
                'verbose_name': 'Smart Filter: Compliance',
                'verbose_name_plural': 'Smart Filter: Compliance',
                'default_permissions': (),
            },
        ),
    ]
