# Generated by Django 4.2.18 on 2025-02-26 14:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('charlink', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AppSettings',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app_name', models.CharField(max_length=255, unique=True)),
                ('ignored', models.BooleanField(default=False)),
                ('default_selection', models.BooleanField(default=True)),
            ],
            options={
                'default_permissions': (),
            },
        ),
        migrations.AlterModelOptions(
            name='general',
            options={'default_permissions': (), 'managed': False, 'permissions': (('view_corp', 'Can view linked character of members of their corporation.'), ('view_alliance', 'Can view linked character of members of their alliance.'), ('view_state', 'Can view linked character of members of their auth state.'), ('view_admin', 'Can view CharLink Admin page.'))},
        ),
    ]
